
'use strict';

let chai = require('chai');
let cp = require('child_process');
let fs = require('fs');
let path = require('path');
let rm = require('rimraf').sync;
let vm = require('vm');

chai.use(require('chai-as-promised'));
let assert = chai.assert;
let fixture = path.resolve.bind(path, __dirname, 'fixtures');


describe('mako cli', function () {
  afterEach(function () {
    rm(fixture('*/build'));
    rm(fixture('*/mako.cache'));
  });

  it('should build a simple project', function () {
    return run('simple').then(() => {
      let file = fixture('simple/build/index.js');
      assert.isTrue(fs.existsSync(file));
      assert.strictEqual(exec(fs.readFileSync(file), path.basename(file)), true);
    });
  });

  it('should not save a cached tree', function () {
    return run('simple').then(() => {
      let file = fixture('simple/mako.cache');
      assert.isFalse(fs.existsSync(file));
    });
  });

  it('should enable the cache', function () {
    return run('simple', '--cache').then(() => {
      let file = fixture('simple/mako.cache');
      assert.isTrue(fs.existsSync(file));
      assert.doesNotThrow(() => JSON.parse(fs.readFileSync(file, 'utf8')));
    });
  });

  it('should expand glob patterns in the config', function () {
    return run('entry-globs').then(() => {
      let files = [
        fixture('entry-globs/build/a.js'),
        fixture('entry-globs/build/b.js'),
        fixture('entry-globs/build/c.js')
      ];

      files.forEach(function (file) {
        assert.isTrue(fs.existsSync(file));
        assert.strictEqual(exec(fs.readFileSync(file), path.basename(file)), path.basename(file, '.js'));
      });
    });
  });

  it('should only run for the entries specified as arguments', function () {
    return run('entry-globs', 'a.js').then(() => {
      let yes = [
        fixture('entry-globs/build/a.js')
      ];

      let no = [
        fixture('entry-globs/build/b.js'),
        fixture('entry-globs/build/c.js')
      ];

      yes.forEach(file => assert.isTrue(fs.existsSync(file)));
      no.forEach(file => assert.isFalse(fs.existsSync(file)));
    });
  });

  it('should allow passing config to plugins', function () {
    return run('plugin-config').then(() => {
      // config excludes write plugin, so output files should not be made
      let file = fixture('plugin-config/build/index.js');
      assert.isFalse(fs.existsSync(file));
    });
  });

  it('should fail when config does not exist', function () {
    return assert.isRejected(run('alternate-config'));
  });

  it('should allow specifying another config path', function () {
    return run('alternate-config', '--config build.json');
  });
});

/**
 * Runs the given fixture through the CLI.
 *
 * @param {String} name   The fixture name.
 * @param {Array} [args]  Additional command arguments.
 * @return {Promise}
 */
function run(name, args) {
  return new Promise(function (resolve, reject) {
    let cmd = path.resolve(__dirname, '../bin/mako');
    if (args) cmd += ' ' + args;
    let options = { cwd: fixture(name) };
    cp.exec(cmd, options, function (err, stdout, stderr) {
      if (err) {
        reject(err);
      } else {
        resolve({ out: stdout, err: stderr });
      }
    });
  });
}

/**
 * Executes the given code, returning it's return value.
 *
 * @param {String} code  The source code to run. (ie: the result of a build)
 * @param {String} id    The file id so it can be run.
 * @return {*}
 */
function exec(code, id) {
  return vm.runInNewContext(code)(id);
}
